/* (C) 2002 by Artem Dmytrenko

   Permission is given to anyone affiliated with 
   ICFP Programming Contest to publish some or all 
   of the source herein in any way shape or form
*/


#include "game.h"

int board_width = 0;
int board_height = 0;

char **board = NULL;
int **dist_board = NULL; 

int bot_id = -1;
int bot_capacity = -1;
int bot_money = -1;

struct package
{
  int id;
  int pos_x;
  int pos_y;
  int dest_x;
  int dest_y;
  int weight;
  
  int on_board;
}; 

struct robot
{
  int id;
  int pos_x;
  int pos_y;
};

struct target
{
  int pos_x;
  int pos_y;
  int distance_to;
  char direction_to;
  
  struct target *next;
};

/* packages available for pickup or are on-board of the friendly robot */
/* we can sacrifice 280k of ram for the sake of simplicity */
struct package packs[10000];

struct robot *bots = NULL;
int number_of_bots = 0;

int start_game()
{
  int i = 0;
  /* initialize packs array */
  for (i = 0; i < 10000; i++)
    packs[i].id = -1;

  if (!write_line("Player\n"))
    return 0;
  
  if (!get_board())
  {
    printf("Unable to get board from server\n");
    return 0;
  }

  if (!get_bot_config())
  {
    printf("Unable to get robot config\n");
    return 0;
  }

  print_board();

  while (1)
  {
    if (!process_move())
    {
      printf("Unable to process move\n");
      return 0;
    }
    
    if (!get_package_list())
    {
      printf("Unable to get package list\n");
      return 0;
    }

    if (!generate_move())
    {
      printf("Could  make a move");
      return 0;
    }
  }

  clean_up();

  return 1;
}

int get_board()
{
  char *buffer = NULL;
  int j;
  /* get board dimensions */

  if ((buffer = read_line()) == NULL)
    return 0;

  if (sscanf(buffer, "%i %i ", &board_width, &board_height) != 2)
    return 0;

  printf ("Board: %ix%i\n", board_width, board_height);

  /* surround map with walls -> two advantages:
        coordinates map properly;
        never have to check for boundaries (speed up);
  */

  board_width += 2;
  board_height += 2;

  if ((board = allocate_board()) == NULL)
    return 0;  

  if ((dist_board = allocate_dist_board()) == NULL)
    return 0;

  memset(board[0], '#', board_width);
  memset(board[board_height - 1], '#', board_width);

  /* how the hell did they manage to flip the board ? */
  for (j = 1; j < board_height - 1; j++)
  {
    board[j][0] = '#';

    /* -1 because there are trailing spaces on all lines */
    if ((buffer = read_line()) == NULL ||
	strlen(buffer) - 1 != board_width - 2)
      return 0;

 
    strcpy(board[j] + 1, buffer);

    board[j][board_width - 1] = '#';
  }

  return 1;
}

int get_bot_config()
{
  char *buffer = NULL;

  if ((buffer = read_line()) == NULL)
    return 0;

  if (sscanf(buffer, "%i %i %i", 
	     &bot_id, &bot_capacity, &bot_money) != 3)
    return 0;

  return 1;
}

int process_move()
{
  char *buffer = NULL;
  int new_number_of_bots = 0;
  struct robot *new_bots = NULL;
  int i = 0;

  if ((buffer = read_line()) == NULL)
    return 0;

  new_number_of_bots = count_bots(buffer);

  if (number_of_bots != new_number_of_bots)
    printf("Number of robots changed: new count = %i\n",
	   new_number_of_bots);

  if (new_number_of_bots != 0)
  {
    if ((new_bots = (struct robot*) 
	 malloc(sizeof(struct robot) * new_number_of_bots)) == NULL)
    {
      printf("Unable to allocate memory for robot array\n");
      return 0;
    }

    /* process every bot's move */
    for (i = 0; i < new_number_of_bots; i++)
    {
      char *next_word;
      int bot_index = -1;
      int local_bot_id;
      int j = 0;

      /* get bot's id */
      if ((next_word = index(buffer, ' ')) == NULL)
      {
	printf("Malformed server's reply\n");
	return 0;
      }
      *next_word = '\0';
      if (sscanf(buffer, "#%i", &local_bot_id) != 1)
      {
	printf("Malformed server's reply\n");
	return 0;
      }

      new_bots[i].id = local_bot_id;
      
      /* increment buffer so we don't read the same info again */
      buffer = next_word + 1;
      
      /* if the bot is in the robots array, find its id */
      for (j = 0; j < number_of_bots; j++) 
	if (bots[j].id == local_bot_id)
	{
	  bot_index = j;

	  new_bots[i].pos_x = bots[j].pos_x;
	  new_bots[i].pos_y = bots[j].pos_y;
	  break;
	}

      /* now process command */
      while(*buffer != '#' && *buffer != '\0') 
	/* there may be multiple commands (insert and move) */
	switch(*buffer)
	{
          case 'N' :
	    {
	      if (bot_index == -1)
	      {
		printf("Bot id=%i does not exist in bot array\n" , local_bot_id);
		return 0;
	      }
	      else
	      {
		new_bots[i].id = bots[bot_index].id;
		new_bots[i].pos_x = bots[bot_index].pos_x;
		new_bots[i].pos_y = bots[bot_index].pos_y + 1;
	      }

	      buffer += 2;
	    }
	    break;

          case 'S' :
	    {
	      if (bot_index == -1)
	      {
		printf("Bot id=%i does not exist in bot array\n" , local_bot_id);
		return 0;
	      }
	      else
	      {
		new_bots[i].id = bots[bot_index].id;
		new_bots[i].pos_x = bots[bot_index].pos_x;
		new_bots[i].pos_y = bots[bot_index].pos_y - 1;
	      }

	      buffer += 2;
	    }
	    break;

	  case 'E' :
	    {
	      if (bot_index == -1)
	      {
		printf("Bot id=%i does not exist in bot array\n" , local_bot_id);
		return 0;
	      }
	      else
	      {
		new_bots[i].id = bots[bot_index].id;
		new_bots[i].pos_x = bots[bot_index].pos_x + 1;
		new_bots[i].pos_y = bots[bot_index].pos_y;
	      }

	      buffer += 2;
	    }
	    break;

          case 'W' :
	    {
	      if (bot_index == -1)
	      {
		printf("Bot id=%i does not exist in bot array\n" , local_bot_id);
		return 0;
	      }
	      else
	      {
		new_bots[i].id = bots[bot_index].id;
		new_bots[i].pos_x = bots[bot_index].pos_x - 1;
		new_bots[i].pos_y = bots[bot_index].pos_y;
	      }

	      buffer += 2;
	    }
	    break;

          case 'P' :
	    {
	      int pack_id = -1;

	      buffer += 2;
	      if ((next_word = index(buffer, ' ')) == NULL)
	      {
		printf ("Could not get package id\n");
		return 0;
	      }

	      *(next_word) = '\0';

	      if (sscanf(buffer, "%i", &pack_id) != 1)
	      {
		printf("Invalid package id\n");
		return 0;
	      }
	      
	      for (j = 0; j < 10000; j++)
		if (packs[j].id == pack_id)
		{
		  if (bot_id == local_bot_id)
		    packs[j].on_board = 1;
		  else
		    /* if hostlie robot picked up pack, black list it */
		    packs[j].id = -1;
		}

	      buffer = next_word + 1;
	    }
	    break;

          case 'D' :
	    {
	      int pack_id = -1;
	      int pack_found = 0;

	      buffer += 2;
	      if ((next_word = index(buffer, ' ')) == NULL)
	      {
		printf ("Could not get package id\n");
		return 0;
	      }

	      *(next_word) = '\0';

	      if (sscanf(buffer, "%i", &pack_id) != 1)
	      {
		printf("Invalid package id\n");
		return 0;
	      }
	      
	      for (j = 0; j < 10000; j++)
		if (packs[j].id == pack_id && packs[j].on_board
		    && bot_id == local_bot_id)
		{
		  /* if delivered to destination, drop */
		  if (packs[j].dest_x == new_bots[i].pos_x &&
		      packs[j].dest_y == new_bots[i].pos_y)
		    packs[j].id = -1;
		  else
		  {
		    /* otherwise, mark the spot as prospective visited base */
		    packs[j].on_board = 0;
		    board[new_bots[i].pos_y][new_bots[i].pos_x] = 'V';
		  }

		  pack_found = 1;
		}

	      /* other bot must have dropped package */
	      /* mark as drop <- possibly come back for it at the end of the game */
	      if (!pack_found)
		board[new_bots[i].pos_y][new_bots[i].pos_x] = 'P';

	      buffer = next_word + 1;
	    }
	    break;

          case 'X' :
	    {
	      /* skip four spaces ahead */
	      next_word = buffer;
	      for (j = 0; j < 4; j++)
		if ((next_word = index(next_word, ' ')) != NULL)
		  next_word++;
		else
		{
		  printf("Invalid position specified\n");
		  return 0;
		}

	      *(next_word - 1) = '\0';
	      
	      if (sscanf(buffer, "X %i Y %i", 
			 &(new_bots[i].pos_x), &(new_bots[i].pos_y)) != 2)
	      {
		printf("Insert bot failed\n");
		return 0;
	      }
	      
	      printf("Insert bot: id=%i at (%i, %i)\n", new_bots[i].id,
		     new_bots[i].pos_x, new_bots[i].pos_y);

	      buffer = next_word;
	    }
	  break;

          default :
	    printf("Invalid action ([%s])\n", buffer);
	    return 0;
	}
    }
  }
  else
    printf("Game ended - no more bots left\n");

  if (bots != NULL)
    free(bots);

  bots = new_bots;
  number_of_bots = new_number_of_bots;

  return 1;
}

int count_bots(char *buffer)
{
  int i = 0;
  int count = 0;
  int len = strlen(buffer);
  
  for (i = 0; i < len; i++)
    if (buffer[i] == '#')
      count++;

  return count;
}

int insert_pack(struct package pack)
{
  int i = 0;
  int insert_idx = 0;

  /* check that id is unique */
  for (i = 0; i < 10000; i++)
    if (pack.id == packs[i].id)
    {
      /* printf("Package with id=%i exists in package array", pack.id); */
      return 0;
    }

  printf("Insert package, id=%i\n", pack.id);
  /* find empty slot */
  for (i = 0; i < 10000; i++)
    if (packs[i].id == -1)
    {
      insert_idx = i;
      break;
    }

  memcpy(&(packs[insert_idx]), &pack, sizeof(struct package));

  return 1;
}

int remove_pack(int id)
{
  int i = 0;

  /* find the id */
  for (i = 0; i < 10000; i++)
    if (packs[i].id == id)
    {
      packs[i].id = -1;
      return 1;
    }

  return 0;
}

int get_package_list()
{
  char *buffer = NULL;
  char *next_word = NULL;

  int bot_pos_x = 0;
  int bot_pos_y = 0;
  int bot_alive = 0;

  int i = 0;

  struct package pack;

  if ((buffer = read_line()) == NULL)
    return 0;
  
  /* find our bot in bot array, and get his position */
  for (i = 0; i < number_of_bots; i++)
    if (bots[i].id == bot_id)
    {
      bot_pos_x = bots[i].pos_x;
      bot_pos_y = bots[i].pos_y;
      bot_alive = 1;
      break;
    }

  if (!bot_alive)
  {
    printf("Delivery bot has died\n");
    return 0;
  }

  printf("Current position: (%i, %i)\n", bot_pos_x, bot_pos_y);

  /* if this is a homebase, mark it as visited */
  if (board[bot_pos_y][bot_pos_x] == '@')
    board[bot_pos_y][bot_pos_x] = 'V';

  /* if empty, continue */
  if (strlen(buffer) == 1)
    return 1;

  if (strncmp(buffer, "Robot", 5) == 0)
  {
    printf("Robot died. Game over.\n");
    return 0;
  }

  
  while(*buffer != '\0')
  {
    if ((next_word = index(buffer, ' ')) == NULL)
      return 0;
    *next_word = '\0';

    if (sscanf(buffer, "%i", &(pack.id)) != 1)
      return 0;

    buffer = next_word + 1;

    if ((next_word = index(buffer, ' ')) == NULL)
      return 0;
    *next_word = '\0';

    if (sscanf(buffer, "%i", &(pack.dest_x)) != 1)
      return 0;

    buffer = next_word + 1;

    if ((next_word = index(buffer, ' ')) == NULL)
      return 0;
    *next_word = '\0';

    if (sscanf(buffer, "%i", &(pack.dest_y)) != 1)
      return 0;

    buffer = next_word + 1;

    if ((next_word = index(buffer, ' ')) == NULL)
      return 0;
    *next_word = '\0';

    if (sscanf(buffer, "%i", &(pack.weight)) != 1)
      return 0;

    buffer = next_word + 1;

    pack.pos_x = bot_pos_x;
    pack.pos_y = bot_pos_y;

    pack.on_board = 0;

    /* insert package if it wasn't there before */
    insert_pack(pack);
  }

  return 1;
}

void flood_fill(int src_x, int src_y, int dest_x, int dest_y, int len)
{
  //printf("Traversing (%i, %i)\n", src_x, src_y);
  //fflush(NULL);

  if (src_x == dest_x && src_y == dest_y)
    return;

  if (dist_board[src_y][src_x] > len)
    dist_board[src_y][src_x] = len;

  /* fill east */
  if (board[src_y][src_x + 1] != '#' &&
      board[src_y][src_x + 1] != '~' &&
      dist_board[src_y][src_x + 1] > len + 1)
    flood_fill(src_x + 1, src_y, dest_x, dest_y, len + 1);

  /* fill north */
  if (board[src_y + 1][src_x] != '#' &&
      board[src_y + 1][src_x] != '~' &&
      dist_board[src_y + 1][src_x] > len + 1)
    flood_fill(src_x, src_y + 1, dest_x, dest_y, len + 1);

  /* fill west */
  if (board[src_y][src_x - 1] != '#' &&
      board[src_y][src_x - 1] != '~' &&
      dist_board[src_y][src_x - 1] > len + 1)
    flood_fill(src_x - 1, src_y, dest_x, dest_y, len + 1);

  /* fill south */
  if (board[src_y - 1][src_x] != '#' &&
      board[src_y - 1][src_x] != '~' &&
      dist_board[src_y - 1][src_x] > len + 1)
    flood_fill(src_x, src_y - 1, dest_x, dest_y, len + 1);
}

struct target* get_closest_target(struct target *targets, int x, int y)
{
  struct target *target_it;
  struct target *closest_target;
  int i, j;

  int dist_n, dist_s, dist_e, dist_w;
  
  if (targets == NULL)
    return NULL;

  target_it = targets;

  while (target_it != NULL)
  {
    /* reset distance map */
    for (j = 0; j < board_height; j++)
      for (i = 0; i < board_width; i++)
	dist_board[j][i] = 10000000;

    /* mark distances on the distance map */
    flood_fill(target_it->pos_x, target_it->pos_y, x, y, 0);

    
    dist_e = dist_board[y][x + 1];
    dist_n = dist_board[y + 1][x];
    dist_w = dist_board[y][x - 1];
    dist_s = dist_board[y - 1][x];

    if (dist_e <= dist_n && dist_e <= dist_w && 
	dist_e <= dist_s)
    {
      target_it->distance_to = dist_e;
      target_it->direction_to = 'E';
    }
    else
      if (dist_n <= dist_w && dist_n <= dist_s)
      {
	target_it->distance_to = dist_n;
	target_it->direction_to = 'N';
      }
      else
	if (dist_w <= dist_s)
	{
	  target_it->distance_to = dist_w;
	  target_it->direction_to = 'W';
	}
	else
	{
	  target_it->distance_to = dist_s;
	  target_it->direction_to = 'S';
	}

    target_it = target_it->next;
  }

  /* find the closest target */
  target_it = targets;
  closest_target = targets;

  while(target_it != NULL)
  {
    if (target_it->distance_to < 
	closest_target->distance_to)
      closest_target = target_it;
    target_it = target_it->next;
  }

  if (closest_target->distance_to < 10000000)
    return closest_target;
  else
    return 0;
}

int generate_move()
{
  int bot_pos_x = 0;
  int bot_pos_y = 0;
  int bot_alive = 0;

  int bots_nearby = 0;

  int max_bet = 50;
  int i = 0;
  int j = 0;
  int k = 0;
  int current_load = 0;

  struct target *targets = NULL;
  struct target *closest_target = NULL;

  /* 60000 byte buffer should accomodate any command */
  /* even when we pick/drop all packages in the same place */ 
  char buffer[60000];
  char package_list[60000];

  memset(buffer, 0, 60000);
  memset(package_list, 0, 60000);


  /* find our bot in bot array, and get his position */
  for (i = 0; i < number_of_bots; i++)
    if (bots[i].id == bot_id)
    {
      bot_pos_x = bots[i].pos_x;
      bot_pos_y = bots[i].pos_y;
      bot_alive = 1;
      break;
    }

  if (!bot_alive)
  {
    printf("Delivery bot has died\n");
    return 0;
  }
  
  /* check if there are other bots around */ 
  /* this function will issue kill command if
     there is a chance we can kill that robot */
  /* kill code = -1 */
  bots_nearby = check_nearby_bots(bot_pos_x, bot_pos_y, max_bet);

  if (bots_nearby == -1)
    return 1;

  /* see if we can drop any packages */
  for (i = 0; i < 10000; i++)
    if (packs[i].id != -1 && packs[i].on_board &&
	packs[i].dest_x == bot_pos_x && packs[i].dest_y == bot_pos_y)
    {
      char pack_id[8];
      sprintf(pack_id, "%i ", packs[i].id);

      strcat(package_list, pack_id);
    }

  /* if we can drop packages, do just that */
  if (strlen(package_list) > 0)
  {
    sprintf(buffer, "1 Drop %s\n", package_list);
    
    if (!write_line(buffer))
    {
      printf("Could not send Drop command\n");
      return 0;
    }

    return 1;
  }

  /* generate current load */
  for (i = 0; i < 10000; i++)
    if (packs[i].id != -1 && packs[i].on_board)
      current_load += packs[i].weight;

  /* see if we can pick up any more packages */
  for (i = 0; i < 10000; i++)
    if (packs[i].id != -1 && !packs[i].on_board &&
	packs[i].pos_x == bot_pos_x && packs[i].pos_y == bot_pos_y &&
	packs[i].weight + current_load <= bot_capacity)
    {
      char pack_id[8];
      sprintf(pack_id, "%i ", packs[i].id);
 
      current_load += packs[i].weight;
      strcat(package_list, pack_id);
    }

  /* if we can pick up packages, do just that */
  if (strlen(package_list) > 0)
  {
    sprintf(buffer, "1 Pick %s\n", package_list);
    if (!write_line(buffer))
    {
      printf("Could not send Pick command\n");
      return 0;
    }
  
    return 1;
  }

  /* create a target list */
  if (current_load == 0)
  {
    struct target *target = targets;

    /* if empty, find a homebase or a lost package */
    for (j = 0; j < board_height; j++)
      for (i = 0; i < board_width; i++)
      {
	int insert_target = 0;

	if (board[j][i] == '@')
	  insert_target = 1;
	else
	  if (board[j][i] == 'V')
	    /* verify that there is at least one package there */
	    for (k = 0; k < 10000; k++)
	      if (packs[k].id != -1 && !packs[k].on_board &&
		  packs[k].pos_x == i && packs[k].pos_y == j)
	      {
		insert_target = 1;
		break;

	      }

	if (insert_target)
	{
	  struct target *new_target;
	  if ((new_target = 
	       (struct target *) malloc (sizeof(struct target))) == NULL)
	  {
	    printf("Cannot allocate memory for target array\n");
	    return 0;
	  }

	  memset(new_target, 0, sizeof(struct target));
	  
	  new_target->pos_x = i;
	  new_target->pos_y = j;
	  
	  if (target != NULL)
	    target->next = new_target;
	  else
	    targets = new_target;
	  
	  target = new_target;
	}
      }
  }
  else
  {
    struct target *target = targets;

    /* if full or partially full, deliver the package */
    for (k = 0; k < 10000; k++)
      if (packs[k].id != -1 && packs[k].on_board)
      {
	int insert_target = 1;

	struct target *target_it = targets;
	
	/* do not insert target if we have another target with
	   the same destinations */
	while (target_it != NULL)
	{
	  if (target_it->pos_x == packs[k].dest_x &&
	      target_it->pos_y == packs[k].dest_y)
	  {
	    insert_target = 0;
	    break;
	  }

	  target_it = target->next;
	}

	if (insert_target)
	{
	  struct target *new_target;
	  if ((new_target = 
	       (struct target *) malloc (sizeof(struct target))) == NULL)
	  {
	    printf("Cannot allocate memory for target array\n");
	    return 0;
	  }

	  memset(new_target, 0, sizeof(struct target));
	  
	  new_target->pos_x = packs[k].dest_x;
	  new_target->pos_y = packs[k].dest_y;
	  
	  if (target != NULL)
	    target->next = new_target;
	  else
	    targets = new_target;
	  
	  target = new_target;
	}
      }
  }

  closest_target = get_closest_target(targets, bot_pos_x, bot_pos_y);

  if (closest_target == NULL)
  {
    int pot_found = 0;
    printf("WARNING: implement self_kill mode\n");

    /* seek for potential drop targets */
    for (j = 0; j < board_width; j++)
      for (i = 0; i < board_height; i++)
	if (board[j][i] == 'V')
	  board[j][i] = '.';
	else
	  if (board[j][i] == 'P')
	  {
	    board[j][i] = '@';
	    pot_found = 1;
	  }
    /* if potential targets were not found, kill ourselves to save money*/
    if (!pot_found)
    {
      printf ("Initializing self destruct\n");
      for (j = 0; j < board_width; j++)
	for (i = 0; i < board_height; i++)
	  if (board[j][i] == '@' || board[j][i] == 'V')
	    board[j][i] = '.';
	  else
	    if (board[j][i] == '~')
	      board[j][i] = '@';
    }

    /* skip a turn anyway */ 
    sprintf(buffer, "1 Pick\n");
  }
  else
    sprintf(buffer, "1 Move %c\n", closest_target->direction_to);

  if (targets != NULL)
  {
    struct target *target_it = targets;

    while(target_it != NULL)
    {
      struct target *tmp_it = target_it->next;
      
      free(target_it);
      target_it = tmp_it;
    }

    targets = NULL;
  }
  
  if (!write_line(buffer))
  {
    printf("Could not send move\n");
    return 0;
  }

  return 1;
}

int bot_present(int x, int y)
{
  int present = 0;
  int i = 0;

  for (i = 0; i < number_of_bots; i++)
    if (bots[i].pos_x == x && bots[i].pos_y == y)
    {
      present = 1;
      break;
    }

  return present;
}

/* returns -1 if kill bot command was issued */
int check_nearby_bots(int x, int y, int kill_bet)
{
  char direction = '\0';
  int kill_try = 0;
  char buffer[1024];

  /* north */
  if (bot_present(x, y + 1) && 
      board[y+2][x] == '~')
  {
    kill_try = 1;
    direction = 'N';
  }
  
  /* east */
  if (bot_present(x + 1, y) &&
      board[y][x+2] == '~')
  {
    kill_try = 1;
    direction = 'E';
  }

  /* south */
  if (bot_present(x, y - 1) && 
      board[y-2][x] == '~')
  {
    kill_try = 1;
    direction = 'S';
  }
  
  /* west */
  if (bot_present(x - 1, y) &&
      board[y][x-2] == '~')
  {
    kill_try = 1;
    direction = 'W';
  }

  if (kill_try)
  {
    printf("Trying to kill hostile bot\n");
    sprintf(buffer, "%i Move %c\n", kill_bet, direction);
    
    if (!write_line(buffer))
    {
      printf("Unable to try to kill other bot\n");
      return 1;
    }
  }
 
  if (kill_try)
    return -1;
  else
    return 0;
}

char **allocate_board()
{
  char **new_board = NULL;
  int j;

  if ((new_board = 
       (char **) malloc(sizeof(char *) * board_height)) == NULL)
  {
    printf("Unable to allocate board\n");
    return NULL;
  }

  for (j = 0; j < board_height; j++)
  {
    if ((new_board[j] = 
	 (char *) malloc(sizeof(char) * board_width)) == NULL)
    {
      printf ("Unable to allocate board\n");
      free(new_board);
      return NULL;
    }
  }

  return new_board;
}

int **allocate_dist_board()
{
  int **new_board = NULL;
  int j;

  if ((new_board = 
       (int **) malloc(sizeof(int *) * board_height)) == NULL)
  {
    printf("Unable to allocate board\n");
    return NULL;
  }

  for (j = 0; j < board_height; j++)
  {
    if ((new_board[j] = 
	 (int *) malloc(sizeof(int) * board_width)) == NULL)
    {
      printf ("Unable to allocate board\n");
      free(new_board);
      return NULL;
    }
  }

  return new_board;
}

void deallocate_board(char **del_board)
{
  int j;

  if (del_board == NULL)
    return;

  for (j = 0; j < board_height; j++)
    if (del_board[j] != NULL)
      free(del_board[j]);
}

void deallocate_dist_board(int **del_board)
{
  int j;

  if (del_board == NULL)
    return;

  for (j = 0; j < board_height; j++)
    if (del_board[j] != NULL)
      free(del_board[j]);
}

void clean_up()
{
  deallocate_board(board);
  board = NULL;

  deallocate_dist_board(dist_board);
  board = NULL;
}

void print_board()
{
  // these arrays are not NULL terminated
  int i, j = 0;

  printf("%ix%i Map:\n", board_width, board_height);
  
  for (j = 0; j < board_height; j++)
  {
    for (i = 0; i < board_width; i++)
      printf("%c", board[j][i]);

    printf("\n");
  }
}
