/* (C) 2002 by Artem Dmytrenko

   Permission is given to anyone affiliated with 
   ICFP Programming Contest to publish some or all 
   of the source herein in any way shape or form
*/

#include "game_sockets.h"

int socket_descriptor = 0;

char *buf = NULL;
int buf_size = 0;

int connect_to_game_server(char *hostname, int port)
{
  struct sockaddr_in socket_in;
  struct hostent *server_hostname;

  if ((server_hostname = gethostbyname(hostname)) == 0)
  {
    printf("Unable to resolve game server hostname\n");
    return 0;
  }

  memset(&socket_in, 0, sizeof (struct sockaddr_in));
  socket_in.sin_family = AF_INET;
  socket_in.sin_addr.s_addr = 
    ((struct in_addr *) (server_hostname->h_addr))->s_addr;
  socket_in.sin_port = htons(port);

  if ((socket_descriptor = socket(AF_INET, SOCK_STREAM, 0)) == -1)
  {
    printf("Unable to create socket\n");
    return 0;
  }

  if (connect(socket_descriptor, 
	      (void *)&socket_in, 
	      sizeof(struct sockaddr_in)) == -1)
  {
    printf("Unable to connect to server\n");
    return 0;
  }

  return 1;
}

char *read_line()
{
  int buf_len = 0;

  do 
  {
    if (buf_len >= buf_size - 1)
    {
      if ((buf = (char *) realloc(buf, buf_size + 1024)) == NULL)
      {
	printf("Ran out of memory while reallocating read buffer\n");
	return NULL;
      }

      buf_size += 1024;
    }

    if (recv(socket_descriptor, buf + buf_len++, 1, 0) != 1)
    {
      printf("Server disconnected during read\n");
      return NULL;
    }
  }
  while(*(buf + buf_len - 1) != '\n');

  /* all strings must have a trailing space
     'cause it makes my life easier */
  if (*(buf + buf_len - 2) != ' ')
  {
    *(buf + buf_len - 1) = ' ';
    buf_len++;
  }

  *(buf + buf_len - 1) = '\0';

  return buf;  
}

int write_line(char *buf)
{
  if (send(socket_descriptor, buf, strlen(buf), 0) == -1)
  {
    printf("Server disconnected during write\n");
    return 0;
  }

  return 1;
}

void disconnect_from_game_server()
{
  close(socket_descriptor);
}

void clean_up_read_buffer()
{
  if (buf != NULL)
    free(buf);

  buf = NULL;
}
